from typing import Any, Mapping, Optional
from tkinter import Widget

DIALOG_ICON: str

class Dialog(Widget):
    widgetName: str = ...
    num: int = ...
    def __init__(self, master: Optional[Any] = ..., cnf: Mapping[str, Any] = ..., **kw) -> None: ...
    def destroy(self) -> None: ...
